// Wild Magic Source Code
// David Eberly
// http://www.geometrictools.com
// Copyright (c) 1998-2007
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or (at
// your option) any later version.  The license is available for reading at
// either of the locations:
//     http://www.gnu.org/copyleft/lgpl.html
//     http://www.geometrictools.com/License/WildMagicLicense.pdf
//
// Version: 4.0.0 (2006/06/28)

#include "Wm4GraphicsPCH.h"
#include "Wm4PixelProgram.h"
#include "Wm4PixelProgramCatalog.h"
using namespace Wm4;

WM4_IMPLEMENT_RTTI(Wm4,PixelProgram,Program);
WM4_IMPLEMENT_STREAM(PixelProgram);
WM4_IMPLEMENT_DEFAULT_STREAM(PixelProgram,Program);
WM4_IMPLEMENT_DEFAULT_NAME_ID(PixelProgram,Program);

//----------------------------------------------------------------------------
PixelProgram* PixelProgram::Load (const std::string& rkProgramName,
    const std::string& rkRendererType, char cCommentPrefix)
{
    std::string kFilename = std::string("p_") + rkProgramName +
        std::string(".") + rkRendererType + std::string(".wmsp");

    PixelProgram* pkProgram = WM4_NEW PixelProgram;
    bool bLoaded = Program::Load(kFilename,cCommentPrefix,pkProgram);
    if (!bLoaded)
    {
        WM4_DELETE pkProgram;
        return 0;
    }

    pkProgram->SetName(rkProgramName.c_str());
    PixelProgramCatalog::GetActive()->Insert(pkProgram);
    return pkProgram;
}
//----------------------------------------------------------------------------
PixelProgram::PixelProgram ()
{
}
//----------------------------------------------------------------------------
PixelProgram::~PixelProgram ()
{
    PixelProgramCatalog::GetActive()->Remove(this);
}
//----------------------------------------------------------------------------
